"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MotifItem = void 0;
const tslib_1 = require("tslib");
const Decimal_1 = require("./Decimal");
const abstract_signer_1 = require("@ethersproject/abstract-signer");
const typechain_1 = require("@motif-foundation/asset/dist/typechain");
const addresses_1 = require("./addresses");
const utils_1 = require("./utils");
const tiny_invariant_1 = (0, tslib_1.__importDefault)(require("tiny-invariant"));
class MotifItem {
    constructor(signerOrProvider, chainId, itemAddress, itemExchangeAddress) {
        if (!itemAddress != !itemExchangeAddress) {
            (0, tiny_invariant_1.default)(false, 'Motif Constructor: itemAddress and itemExchangeAddress must both be non-null or both be null');
        }
        if (abstract_signer_1.Signer.isSigner(signerOrProvider)) {
            this.readOnly = false;
        }
        else {
            this.readOnly = true;
        }
        this.signerOrProvider = signerOrProvider;
        this.chainId = chainId;
        if (itemAddress && itemExchangeAddress) {
            const parsedItemAddress = (0, utils_1.validateAndParseAddress)(itemAddress);
            const parsedItemExchangeAddress = (0, utils_1.validateAndParseAddress)(itemExchangeAddress);
            this.itemAddress = parsedItemAddress;
            this.itemExchangeAddress = parsedItemExchangeAddress;
        }
        else {
            const network = (0, utils_1.chainIdToNetworkName)(chainId);
            this.itemAddress = addresses_1.addresses[network].item;
            this.itemExchangeAddress = addresses_1.addresses[network].itemExchange;
        }
        this.item = typechain_1.ItemFactory.connect(this.itemAddress, signerOrProvider);
        this.itemExchange = typechain_1.ItemExchangeFactory.connect(this.itemExchangeAddress, signerOrProvider);
    }
    /*********************
     * Motif View Methods
     *********************
     */
    /**
     * Fetches the content hash for the specified item on the Motif Item Contract
     * @param itemId
     */
    async fetchContentHash(itemId) {
        return this.item.tokenContentHashes(itemId);
    }
    /**
     * Fetches the metadata hash for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async fetchMetadataHash(itemId) {
        return this.item.tokenMetadataHashes(itemId);
    }
    /**
     * Fetches the content uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async fetchContentURI(itemId) {
        return this.item.tokenURI(itemId);
    }
    /**
     * Fetches the metadata uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async fetchMetadataURI(itemId) {
        return this.item.tokenMetadataURI(itemId);
    }
    /**
     * Fetches the creator for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async fetchCreator(itemId) {
        return this.item.tokenCreators(itemId);
    }
    /**
     * Fetches the current bid shares for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async fetchCurrentBidShares(itemId) {
        return this.itemExchange.bidSharesForToken(itemId);
    }
    /**
     * Fetches the current ask for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async fetchCurrentAsk(itemId) {
        return this.itemExchange.currentAskForToken(itemId);
    }
    /**
     * Fetches the current bid for the specified bidder for the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param bidder
     */
    async fetchCurrentBidForBidder(itemId, bidder) {
        return this.itemExchange.bidForTokenBidder(itemId, bidder);
    }
    /**
     * Fetches the permit nonce on the specified item id for the owner address
     * @param address
     * @param itemId
     */
    async fetchPermitNonce(address, itemId) {
        return this.item.permitNonces(address, itemId);
    }
    /**
     * Fetches the current mintWithSig nonce for the specified address
     * @param address
     * @param itemId
     */
    async fetchMintWithSigNonce(address) {
        return this.item.mintWithSigNonces(address);
    }
    /*********************
     * Motif Write Methods
     *********************
     */
    /**
     * Updates the content uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param tokenURI
     */
    async updateContentURI(itemId, tokenURI) {
        try {
            this.ensureNotReadOnly();
            (0, utils_1.validateURI)(tokenURI);
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.updateTokenURI(itemId, tokenURI);
    }
    /**
     * Updates the metadata uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param metadataURI
     */
    async updateMetadataURI(itemId, metadataURI) {
        try {
            this.ensureNotReadOnly();
            (0, utils_1.validateURI)(metadataURI);
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.updateTokenMetadataURI(itemId, metadataURI);
    }
    /**
     * Mints a new piece of item on an instance of the Motif Item Contract
     * @param mintData
     * @param bidShares
     */
    async mint(itemData, bidShares) {
        try {
            this.ensureNotReadOnly();
            (0, utils_1.validateURI)(itemData.metadataURI);
            (0, utils_1.validateURI)(itemData.tokenURI);
            (0, utils_1.validateBidShares)(bidShares.creator, bidShares.owner, bidShares.prevOwner);
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        const gasEstimate = await this.item.estimateGas.mint(itemData, bidShares);
        const paddedEstimate = gasEstimate.mul(110).div(100);
        return this.item.mint(itemData, bidShares, { gasLimit: paddedEstimate.toString() });
    }
    /**
     * Mints a new piece of item on an instance of the Motif Item Contract
     * @param creator
     * @param itemData
     * @param bidShares
     * @param sig
     */
    async mintWithSig(creator, itemData, bidShares, sig) {
        try {
            this.ensureNotReadOnly();
            (0, utils_1.validateURI)(itemData.metadataURI);
            (0, utils_1.validateURI)(itemData.tokenURI);
            (0, utils_1.validateBidShares)(bidShares.creator, bidShares.owner, bidShares.prevOwner);
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.mintWithSig(creator, itemData, bidShares, sig);
    }
    /**
     * Sets an ask on the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param ask
     */
    async setAsk(itemId, ask) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.setAsk(itemId, ask);
    }
    /**
     * Sets a bid on the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param bid
     */
    async setBid(itemId, bid) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.setBid(itemId, bid);
    }
    /**
     * Removes the ask on the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async removeAsk(itemId) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.removeAsk(itemId);
    }
    /**
     * Removes the bid for the msg.sender on the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async removeBid(itemId) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.removeBid(itemId);
    }
    /**
     * Accepts the specified bid on the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param bid
     */
    async acceptBid(itemId, bid) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.acceptBid(itemId, bid);
    }
    /**
     * Grants the spender approval for the specified item using meta transactions as outlined in EIP-712
     * @param sender
     * @param itemId
     * @param sig
     */
    async permit(spender, itemId, sig) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.permit(spender, itemId, sig);
    }
    /**
     * Revokes the approval of an approved account for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async revokeApproval(itemId) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.revokeApproval(itemId);
    }
    /**
     * Burns the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async burn(itemId) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.burn(itemId);
    }
    /***********************
     * ERC-721 View Methods
     ***********************
     */
    /**
     * Fetches the total balance of item owned by the specified owner on an instance of the Motif Item Contract
     * @param owner
     */
    async fetchBalanceOf(owner) {
        return this.item.balanceOf(owner);
    }
    /**
     * Fetches the owner of the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async fetchOwnerOf(itemId) {
        return this.item.ownerOf(itemId);
    }
    /**
     * Fetches the itemId of the specified owner by index on an instance of the Motif Item Contract
     * @param owner
     * @param index
     */
    async fetchItemOfOwnerByIndex(owner, index) {
        return this.item.tokenOfOwnerByIndex(owner, index);
    }
    /**
     * Fetches the total amount of non-burned item that has been minted on an instance of the Motif Item Contract
     */
    async fetchTotalItem() {
        return this.item.totalSupply();
    }
    /**
     * Fetches the itemId by index on an instance of the Motif Item Contract
     * @param index
     */
    async fetchItemByIndex(index) {
        return this.item.tokenByIndex(index);
    }
    /**
     * Fetches the approved account for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    async fetchApproved(itemId) {
        return this.item.getApproved(itemId);
    }
    /**
     * Fetches if the specified operator is approved for all item owned by the specified owner on an instance of the Motif Item Contract
     * @param owner
     * @param operator
     */
    async fetchIsApprovedForAll(owner, operator) {
        return this.item.isApprovedForAll(owner, operator);
    }
    /***********************
     * ERC-721 Write Methods
     ***********************
     */
    /**
     * Grants approval to the specified address for the specified item on an instance of the Motif Item Contract
     * @param to
     * @param itemId
     */
    async approve(to, itemId) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.approve(to, itemId);
    }
    /**
     * Grants approval for all item owner by msg.sender on an instance of the Motif Item Contract
     * @param operator
     * @param approved
     */
    async setApprovalForAll(operator, approved) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.setApprovalForAll(operator, approved);
    }
    /**
     * Transfers the specified item to the specified to address on an instance of the Motif Item Contract
     * @param from
     * @param to
     * @param itemId
     */
    async transferFrom(from, to, itemId) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.transferFrom(from, to, itemId);
    }
    /**
     * Executes a SafeTransfer of the specified item to the specified address if and only if it adheres to the ERC721-Receiver Interface
     * @param from
     * @param to
     * @param itemId
     */
    async safeTransferFrom(from, to, itemId) {
        try {
            this.ensureNotReadOnly();
        }
        catch (err) {
            return Promise.reject(err.message);
        }
        return this.item.safeTransferFrom(from, to, itemId);
    }
    /****************
     * Miscellaneous
     * **************
     */
    /**
     * Returns the EIP-712 Domain for an instance of the Motif Item Contract
     */
    eip712Domain() {
        // Due to a bug in ganache-core, set the chainId to 1 if its a local blockchain
        // https://github.com/trufflesuite/ganache-core/issues/515
        const chainId = this.chainId == 50 ? 1 : this.chainId;
        return {
            name: 'Motif',
            version: '1',
            chainId: chainId,
            verifyingContract: this.itemAddress,
        };
    }
    /**
     * Checks to see if a Bid's amount is evenly splittable given the item's current bidShares
     *
     * @param itemId
     * @param bid
     */
    async isValidBid(itemId, bid) {
        const isAmountValid = await this.itemExchange.isValidBid(itemId, bid.amount);
        const decimal100 = Decimal_1.Decimal.new(100);
        const currentBidShares = await this.fetchCurrentBidShares(itemId);
        const isSellOnShareValid = bid.sellOnShare.value.lte(decimal100.value.sub(currentBidShares.creator.value));
        return isAmountValid && isSellOnShareValid;
    }
    /**
     * Checks to see if an Ask's amount is evenly splittable given the item's current bidShares
     *
     * @param itemId
     * @param ask
     */
    isValidAsk(itemId, ask) {
        return this.itemExchange.isValidBid(itemId, ask.amount);
    }
    /**
     * Checks to see if a piece of item has verified uris that hash to their immutable hashes
     *
     * @param itemId
     * @param timeout
     */
    async isVerifiedItem(itemId, timeout = 10) {
        try {
            const [tokenURI, metadataURI, contentHash, metadataHash] = await Promise.all([
                this.fetchContentURI(itemId),
                this.fetchMetadataURI(itemId),
                this.fetchContentHash(itemId),
                this.fetchMetadataHash(itemId),
            ]);
            const itemData = (0, utils_1.constructItemData)(tokenURI, metadataURI, contentHash, metadataHash);
            return (0, utils_1.isItemDataVerified)(itemData, timeout);
        }
        catch (err) {
            return Promise.reject(err.message);
        }
    }
    /******************
     * Private Methods
     ******************
     */
    /**
     * Throws an error if called on a readOnly == true instance of Motif Sdk
     * @private
     */
    ensureNotReadOnly() {
        if (this.readOnly) {
            throw new Error('ensureNotReadOnly: readOnly Motif instance cannot call contract methods that require a signer.');
        }
    }
}
exports.MotifItem = MotifItem;
//# sourceMappingURL=MotifItem.js.map